import React, { useState } from "react";
import { Card, CardHeader, CardTitle, CardContent } from "@/components/ui/card";
import { Button } from "@/components/ui/button";
import { Input } from "@/components/ui/input";
import Select from "react-select";
import { CalendarIcon, Search } from "lucide-react";
import { format } from "date-fns";
import { DateRange } from "react-day-picker";
import {
    Popover,
    PopoverContent,
    PopoverTrigger,
} from "@/components/ui/popover";
import { Calendar as UIDatePicker } from "@/components/ui/calendar";
import {
    Perusahaan,
    JenisDokIL,
    JenisKegiatan,
    PerizinanLingkunganType,
} from "@/types/perusahaan";

type SelectOption = {
    value: string;
    label: string;
};

type FilterPerizinanLingkunganProps = {
    perusahaan?: Perusahaan[];
    jenisDokIL?: JenisDokIL[];
    jenisKegiatan?: JenisKegiatan[];
    izinLingkungan?: PerizinanLingkunganType[];
    onSearch: (filters: {
        selectedPerusahaan: SelectOption | null;
        selectedJenisKegiatan: SelectOption | null;
        Alamat: string;
        TeleponFax: string;
        Email: string;
        selectedJenisIzin: SelectOption | null;
        ILNomor: string;
        ILTanggal: string | null;
    }) => void;
};

export default function FilterPerizinanLingkungan({
    perusahaan = [],
    jenisDokIL = [],
    jenisKegiatan = [],
    onSearch,
}: FilterPerizinanLingkunganProps) {
    const [dateRange, setDateRange] = useState<DateRange | undefined>({
        from: undefined,
        to: undefined,
    });

    const perusahaanOptions: SelectOption[] = perusahaan.map((p) => ({
        value: p.PerusahaanId.toString(),
        label: p.NamaPerusahaan,
    }));

    const jenisKegiatanOptions: SelectOption[] = jenisKegiatan.map((jk) => ({
        value: jk.JenisKegiatanId.toString(),
        label: jk.NamaJenisKegiatan,
    }));

    const jenisIzinOptions: SelectOption[] = jenisDokIL.map((jdi) => ({
        value: jdi.JenisDokILId.toString(),
        label: jdi.NamaJenisDokIL,
    }));

    const selectStyles = {
        control: (base: any) => ({
            ...base,
            minHeight: "38px",
            borderRadius: "0.375rem",
            borderColor: "#e2e8f0",
            boxShadow: "none",
            "&:hover": {
                borderColor: "#cbd5e0",
            },
        }),
    };

    const [selectedPerusahaan, setSelectedPerusahaan] =
        useState<SelectOption | null>(null);
    const [selectedJenisKegiatan, setSelectedJenisKegiatan] =
        useState<SelectOption | null>(null);
    const [Alamat, setAlamat] = useState<string>("");
    const [TeleponFax, setTeleponFax] = useState<string>("");
    const [Email, setEmail] = useState<string>("");
    const [selectedJenisIzin, setSelectedJenisIzin] =
        useState<SelectOption | null>(null);
    const [ILNomor, setILNomor] = useState<string>("");
    const [ILTanggal, setILTanggal] = useState<Date | null>(null);

    function handleSearch() {
        let formattedDate: string | null = null;
        if (dateRange?.from && dateRange?.to) {
            // Jika kedua tanggal dipilih, format sebagai rentang
            formattedDate = `${format(dateRange.from, "yyyy-MM-dd")} - ${format(
                dateRange.to,
                "yyyy-MM-dd"
            )}`;
        } else if (dateRange?.from) {
            // Jika hanya tanggal mulai yang dipilih, gunakan tanggal tersebut
            formattedDate = format(dateRange.from, "yyyy-MM-dd");
        }
        onSearch({
            selectedPerusahaan,
            selectedJenisKegiatan,
            Alamat,
            TeleponFax,
            Email,
            selectedJenisIzin,
            ILNomor,
            ILTanggal: formattedDate,
        });
    }
    function handleReset() {
        // Reset all form state
        setDateRange({ from: undefined, to: undefined });
        setSelectedPerusahaan(null);
        setSelectedJenisKegiatan(null);
        setAlamat("");
        setTeleponFax("");
        setEmail("");
        setSelectedJenisIzin(null);
        setILNomor("");
        setILTanggal(null);

        // Reset the search results by calling onSearch with empty/default values
        onSearch({
            selectedPerusahaan: null,
            selectedJenisKegiatan: null,
            Alamat: "",
            TeleponFax: "",
            Email: "",
            selectedJenisIzin: null,
            ILNomor: "",
            ILTanggal: null,
        });
    }

    return (
        <div className="bg-white py-6 rounded-lg shadow-sm mb-6">
            <Card className="shadow-md border-slate-200">
                <CardHeader className="pb-3">
                    <CardTitle className="text-lg font-medium">
                        Filter Pencarian Perizinan Lingkungan
                    </CardTitle>
                </CardHeader>
                <CardContent>
                    <div className="grid grid-cols-1 md:grid-cols-2 gap-8">
                        {/* Kolom Kiri */}
                        <div className="space-y-4">
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Nama Perusahaan
                                </label>
                                <div className="col-span-2">
                                    <Select
                                        options={perusahaanOptions}
                                        value={selectedPerusahaan}
                                        onChange={setSelectedPerusahaan}
                                        placeholder="Pilih Perusahaan"
                                        isClearable
                                        isSearchable
                                        styles={selectStyles}
                                        noOptionsMessage={() =>
                                            "Tidak ada data"
                                        }
                                    />
                                </div>
                            </div>
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Jenis Kegiatan
                                </label>
                                <div className="col-span-2">
                                    <Select
                                        options={jenisKegiatanOptions}
                                        value={selectedJenisKegiatan}
                                        onChange={setSelectedJenisKegiatan}
                                        placeholder="Pilih Jenis Kegiatan"
                                        isClearable
                                        isSearchable
                                        styles={selectStyles}
                                        noOptionsMessage={() =>
                                            "Tidak ada data"
                                        }
                                    />
                                </div>
                            </div>
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Alamat
                                </label>
                                <div className="col-span-2">
                                    <Input
                                        placeholder="Masukkan Alamat"
                                        value={Alamat}
                                        onChange={(e) =>
                                            setAlamat(e.target.value)
                                        }
                                    />
                                </div>
                            </div>
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Telepon/Fax
                                </label>
                                <div className="col-span-2">
                                    <Input
                                        placeholder="Masukkan Telepon/Fax"
                                        value={TeleponFax}
                                        onChange={(e) =>
                                            setTeleponFax(e.target.value)
                                        }
                                    />
                                </div>
                            </div>
                        </div>
                        {/* Kolom Kanan */}
                        <div className="space-y-4">
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Email
                                </label>
                                <div className="col-span-2">
                                    <Input
                                        placeholder="Masukkan Email"
                                        value={Email}
                                        onChange={(e) =>
                                            setEmail(e.target.value)
                                        }
                                    />
                                </div>
                            </div>
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Jenis Izin
                                </label>
                                <div className="col-span-2">
                                    <Select
                                        options={jenisIzinOptions}
                                        value={selectedJenisIzin}
                                        onChange={setSelectedJenisIzin}
                                        placeholder="Pilih Jenis Izin"
                                        isClearable
                                        isSearchable
                                        styles={selectStyles}
                                        noOptionsMessage={() =>
                                            "Tidak ada data"
                                        }
                                    />
                                </div>
                            </div>
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Nomor Izin
                                </label>
                                <div className="col-span-2">
                                    <Input
                                        placeholder="Masukkan Nomor Izin"
                                        value={ILNomor}
                                        onChange={(e) =>
                                            setILNomor(e.target.value)
                                        }
                                    />
                                </div>
                            </div>
                            <div className="grid grid-cols-3 items-center gap-4">
                                <label className="text-sm font-medium text-gray-700">
                                    Rentang Tanggal Izin
                                </label>
                                <div className="col-span-2">
                                    <Popover>
                                        <PopoverTrigger asChild>
                                            <Button
                                                variant="outline"
                                                className="w-full text-left"
                                            >
                                                <CalendarIcon className="mr-2 h-4 w-4" />
                                                {dateRange?.from
                                                    ? dateRange.to
                                                        ? `${format(
                                                              dateRange.from,
                                                              "dd/MM/yyyy"
                                                          )} - ${format(
                                                              dateRange.to,
                                                              "dd/MM/yyyy"
                                                          )}`
                                                        : format(
                                                              dateRange.from,
                                                              "dd/MM/yyyy"
                                                          )
                                                    : "Pilih Rentang Tanggal"}
                                            </Button>
                                        </PopoverTrigger>
                                        <PopoverContent
                                            className="w-auto p-0"
                                            align="start"
                                        >
                                            <UIDatePicker
                                                initialFocus
                                                mode="range"
                                                defaultMonth={dateRange?.from}
                                                selected={dateRange}
                                                onSelect={setDateRange}
                                                numberOfMonths={2}
                                            />
                                        </PopoverContent>
                                    </Popover>
                                </div>
                            </div>
                        </div>
                    </div>
                    <div className="flex justify-end mt-6 gap-2">
                        <Button
                            className="bg-green-600 hover:bg-green-700 text-white"
                            onClick={handleSearch}
                        >
                            <Search className="w-4 h-4 mr-2" />
                            Cari
                        </Button>
                        <Button
                            className="bg-gray-600 hover:bg-gray-700 text-white"
                            onClick={handleReset}
                        >
                            Reset
                        </Button>
                    </div>
                </CardContent>
            </Card>
        </div>
    );
}
